import { pregenerateKeyframes } from './pregenerate-keyframes.es.js';

function createGeneratorEasing(createGenerator) {
    const keyframesCache = new WeakMap();
    return (options = {}) => {
        const generatorCache = new Map();
        const getGenerator = (from = 0, to = 100, velocity = 0, isScale = false) => {
            const key = `${from}-${to}-${velocity}-${isScale}`;
            if (!generatorCache.has(key)) {
                generatorCache.set(key, createGenerator(Object.assign({ from,
                    to,
                    velocity, restSpeed: isScale ? 0.05 : 2, restDistance: isScale ? 0.01 : 0.5 }, options)));
            }
            return generatorCache.get(key);
        };
        const getKeyframes = (generator) => {
            if (!keyframesCache.has(generator)) {
                keyframesCache.set(generator, pregenerateKeyframes(generator));
            }
            return keyframesCache.get(generator);
        };
        return {
            createAnimation: (keyframes, getOrigin, canUseGenerator, name, data) => {
                let settings;
                let generator;
                const numKeyframes = keyframes.length;
                let shouldUseGenerator = canUseGenerator &&
                    numKeyframes <= 2 &&
                    keyframes.every(isNumberOrNull);
                if (shouldUseGenerator) {
                    const prevAnimationState = name && data && data.prevGeneratorState[name];
                    const velocity = prevAnimationState &&
                        (numKeyframes === 1 ||
                            (numKeyframes === 2 && keyframes[0] === null))
                        ? prevAnimationState.velocity
                        : 0;
                    const target = keyframes[numKeyframes - 1];
                    const unresolvedOrigin = numKeyframes === 1 ? null : keyframes[0];
                    const origin = unresolvedOrigin === null
                        ? prevAnimationState
                            ? prevAnimationState.value
                            : parseFloat(getOrigin())
                        : unresolvedOrigin;
                    generator = getGenerator(origin, target, velocity, name === null || name === void 0 ? void 0 : name.includes("scale"));
                    const keyframesMetadata = getKeyframes(generator);
                    settings = Object.assign(Object.assign({}, keyframesMetadata), { easing: "linear" });
                }
                else {
                    generator = getGenerator(0, 100);
                    const keyframesMetadata = getKeyframes(generator);
                    settings = {
                        easing: "ease",
                        duration: keyframesMetadata.overshootDuration,
                    };
                }
                // TODO Add test for this
                if (generator && data && name) {
                    data.generators[name] = generator;
                }
                return settings;
            },
        };
    };
}
const isNumberOrNull = (value) => typeof value !== "string";

export { createGeneratorEasing };
